@@ -22,7 +22,7 @@ fix.dimnames <- function(x) {
 }
 
 #constructor function
-init.sts <- function(.Object, week, start=c(2000,1), freq=52, observed, state, map=NULL, neighbourhood=NULL, populationFrac=NULL,alarm=NULL,upperbound=NULL, control=NULL) {
+init.sts <- function(.Object, epoch, start=c(2000,1), freq=52, observed, state=0*observed, map=NULL, neighbourhood=NULL, populationFrac=NULL,alarm=NULL,upperbound=NULL, control=NULL,epochAsDate=FALSE,multinomialTS=FALSE) {
   #Name handling
   namesObs <-colnames(observed)
   namesState <- colnames(observed)
@@ -53,7 +53,7 @@ init.sts <- function(.Object, week, start=c(2000,1), freq=52, observed, state, m
   if (is.null(populationFrac)) {
     populationFrac <- matrix(1/nAreas,nrow=nObs,ncol=nAreas)
   }
-  if (nAreas ==1){
+  if (nAreas ==1 & (!multinomialTS)){
     populationFrac <- matrix(1,nrow=nObs, ncol=1)
   }
   
@@ -73,8 +73,10 @@ init.sts <- function(.Object, week, start=c(2000,1), freq=52, observed, state, m
   if (is.null(upperbound))
     upperbound <- matrix(NA,nrow=dim(observed)[1],ncol=dim(observed)[2])
 
-  ##Assign everything
-  .Object@week <- week
+  ##Assign everything else
+  .Object@week <- epoch
+  .Object@epochAsDate <- epochAsDate
+  .Object@multinomialTS <- multinomialTS
 
   if (length(start) == 2) {
     .Object@start <- start
@@ -122,7 +124,7 @@ disProg2sts <- function(disProgObj, map=NULL) {
     myweek <- disProgObj$week
   }
     
-  sts <- new("sts", week=myweek, start=disProgObj$start, freq=disProgObj$freq, observed=disProgObj$observed, state = disProgObj$state, map=map, neighbourhood=disProgObj$neighbourhood, populationFrac=disProgObj$populationFrac,alarm=disProgObj$alarm,upperbound=disProgObj$upperbound)
+  sts <- new("sts", epoch=myweek, start=disProgObj$start, freq=disProgObj$freq, observed=disProgObj$observed, state = disProgObj$state, map=map, neighbourhood=disProgObj$neighbourhood, populationFrac=disProgObj$populationFrac,alarm=disProgObj$alarm,upperbound=disProgObj$upperbound)
   return(sts)
 }
 
@@ -130,7 +132,7 @@ disProg2sts <- function(disProgObj, map=NULL) {
 sts2disProg <- function(sts) {
   disProgObj <- create.disProg(week=sts@week, start=sts@start, freq=sts@freq,
                                observed=sts@observed, state=sts@state, neighbourhood=sts@neighbourhood,
-                               populationFrac=sts@populationFrac)
+                               populationFrac=sts@populationFrac, epochAsDate=sts@epochAsDate)
   #For survRes: alarm=sts@alarm, upperbound=sts@upperbound)
   return(disProgObj)
 }
@@ -218,8 +220,24 @@ setMethod("colnames", signature=c(x="sts",do.NULL="missing",prefix="missing"), f
 setGeneric("year", function(x, ...) standardGeneric("year"));
 setMethod("year", "sts", function(x,...) return((x@week-1) %/% x@freq + x@start[1]))
 #Extract which observation within year we have
-setGeneric("obsinyear", function(x, ...) standardGeneric("obsinyear"));
-setMethod("obsinyear", "sts", function(x,...) return( (x@week-1 + x@start[2]-1) %% x@freq + 1))
+setGeneric("epochInYear", function(x, ...) standardGeneric("epochInYear"));
+setMethod("epochInYear", "sts", function(x,...) {
+  if (x@epochAsDate) {
+    epochStr <- switch( as.character(x@freq), "12" = "%m","52" =  "%V","365" = "%j")
+    return(as.numeric(format(epoch(x),epochStr)))
+  } else {
+    return( (x@week-1 + x@start[2]-1) %% x@freq + 1)
+  }
+})
+setGeneric("year", function(x, ...) standardGeneric("year"));
+setMethod("year", "sts", function(x,...) {
+  if (x@epochAsDate) {
+    return(as.numeric(format(epoch(x),"%G")))
+  } else {
+    ((x@week-1 + x@start[2]-1) + (x@freq*x@start[1])) %/% x@freq 
+  }
+})
+
 
 
 #####################################################################
@@ -331,7 +349,100 @@ merge.list <- function (x, y, ...)
 # colors - c( fill color of polygons, line color of polygons, upperbound)
 ##########################################################################
 
-plot.sts.time.one <- function(x, k=1, domany=FALSE,ylim=NULL,xaxis.years=TRUE, xaxis.units=TRUE, xlab="time", ylab="No. infected", main=NULL, type="s",lty=c(1,1,2),col=c(NA,1,4),lwd=c(1,1,1), outbreak.symbol = list(pch=3, col=3, cex=1),alarm.symbol=list(pch=24, col=2, cex=1),cex=1,legend.opts=list(x="top", legend=NULL,lty=NULL,pch=NULL,col=NULL),dx.upperbound=0.5,hookFunc=function() {},...) {
+######################################################################
+#Format especially x-axis according to year and epoch. Also handling
+#ISO weeks.
+#This function could also use plot.Date, but then x-axis
+#has not simple 1:nrow(x) interpretation anymore.
+#
+#myplot <- function(x,...) {
+#  plot(epoch(x),observed(x),xaxt="n",type="h")#,...)
+#  axis.Date(1, x=epoch(x),las=2,format="%Y-W%V",at=seq(min(epoch(x)),max(epoch(x)),by="1 month"),las=2)
+#  axis.Date(1, x=epoch(x),format="%Y-W%V",at=seq(min(epoch(x)),max(epoch(x)),by="3 month"),las=2)
+#  axis.Date(1, x=epoch(x),at=seq(min(epoch(x)),max(epoch(x)),by="year"),label=FALSE,tcl=-1)
+#}
+######################################################################
+
+
+addFormattedXAxis <- function(x, epochsAsDate, observed, firstweek,xaxis.units,cex) {
+  #Declare commonly used variables.
+  startyear <-  x@start[1]
+  
+  if (x@freq ==52) {
+    if (!epochsAsDate) {
+      # At which indices to put the "at" tick label. This will
+      # be exactly those week numbers where the new quarter begins: 1, 14, 27 and 40 + i*52.
+      # Note that week number and index is not the same due to the "firstweek" argument
+      weeks <- 1:length(observed) + (firstweek-1)
+      noYears <- ceiling(max(weeks)/52)
+      quarterStarts <- rep( (0:(noYears))*52, each=4) + rep( c(1,14,27,40), noYears+1)
+      weeks <- subset(weeks, !is.na(match(weeks,quarterStarts)))
+      weekIdx <- weeks - (firstweek-1)
+
+      # get the right year for each week
+      year <- weeks %/% 52 + startyear
+      # function to define the quarter order
+      quarterFunc <- function(i) { switch(i+1,"I","II","III","IV")}
+      # get the right number and order of quarter labels
+      quarter <- sapply( (weeks-1) %/% 13 %% 4, quarterFunc)
+    } else {   #If epochAsDate -- experimental functionality to handle ISO 8601
+      date <- as.Date(x@week, origin="1970-01-01")
+      years <- unique(as.numeric(format(date,"%Y")))
+      #Start of quarters in each year present in the data. 
+      qStart <- as.Date(paste(rep(years,each=4), c("-01-01","-04-01","-07-01","-10-01"),sep=""))
+      qName  <- rep(c("I","II","III","IV"), length.out=length(qStart))
+      qIdx   <- qStart <= max(date)+10 & qStart >= min(date)-10
+      qStart <- qStart[qIdx] ; qName <- qName[qIdx]
+      #Find week in data closest to these dates
+      weekIdx <- sapply(qStart, function(d) which.min(abs(as.numeric(date - d))))
+
+      date <- date[weekIdx]
+      #Year the ISO week belongs to
+      year <- as.numeric(format(date,"%G"))
+      quarter <- qName
+    }        
+      
+    #construct the computed axis labels -- add quarters if xaxis.units is requested
+    if (xaxis.units) {
+      labels.week <- paste(year,"\n\n",quarter,sep="")
+    } else {
+      labels.week <- paste(year,sep="")
+    }
+
+    axis( side=1,line=1,labels=FALSE,at=c(1,length(observed)),lwd.ticks=0)
+    axis( at=weekIdx[which(quarter != "I")] , labels=labels.week[which(quarter != "I")] , side=1, line = 1 ,cex=cex)
+    #Bigger tick marks at the first quarter
+    at <- weekIdx[which(quarter == "I")]
+    axis( at=at  , labels=rep(NA,length(at)), side=1, line = 1 ,tcl=2*par()$tcl)
+    #2nd axis
+#    axis( side=2 ,cex=cex)
+  } else { ##other frequency
+    #A label at each unit
+    myat.unit <- seq(firstweek,length.out=length(observed) )
+
+    # get the right year order
+    month <- (myat.unit-1) %% x@freq + 1
+    year <- (myat.unit - 1) %/% x@freq + startyear
+    #construct the computed axis labels -- add quarters if xaxis.units is requested
+    if (xaxis.units) {
+      mylabels.unit <- paste(year,"\n\n", (myat.unit-1) %% x@freq + 1,sep="")
+    } else {
+      mylabels.unit <- paste(year,sep="")
+    }
+    #Add axis
+    axis( at=(1:length(observed))  , labels=NA, side=1, line = 1 ,cex=cex)
+    axis( at=(1:length(observed))[month==1]  , labels=mylabels.unit[month==1] , side=1, line = 1 ,cex=cex)
+#        axis( at=(1:length(observed)), labels=mylabels.unit, side=1, line = 1 ,cex=cex)
+    #Bigger tick marks at the first unit
+    at <- (1:length(observed))[(myat.unit - 1) %% x@freq == 0]
+    axis( at=at  , labels=rep(NA,length(at)), side=1, line = 1 ,tcl=2*par()$tcl)
+    #2nd axis
+#    axis( side=2 ,cex=cex)
+  }
+  invisible()
+}
+
+plot.sts.time.one <- function(x, k=1, domany=FALSE,ylim=NULL,xaxis.years=TRUE, axes=TRUE, xaxis.units=TRUE, epochsAsDate=x@epochAsDate, xlab="time", ylab="No. infected", main=NULL, type="s",lty=c(1,1,2),col=c(NA,1,4),lwd=c(1,1,1), outbreak.symbol = list(pch=3, col=3, cex=1),alarm.symbol=list(pch=24, col=2, cex=1),cex=1,legend.opts=list(x="top", legend=NULL,lty=NULL,pch=NULL,col=NULL),dx.upperbound=0.5,hookFunc=function() {},...) {
 
   #Extract slots -- depending on the algorithms: x@control$range
   observed   <- x@observed[,k]
@@ -344,11 +455,11 @@ plot.sts.time.one <- function(x, k=1, domany=FALSE,ylim=NULL,xaxis.years=TRUE, x
   method <-     x@control$name
   disease <-    x@control$data
   population <- x@populationFrac[,k]
-  binaryTS <- sum( population > 1 ) > 1
+  binaryTS <- x@multinomialTS
 
   if (binaryTS) {
-    observed <- observed/population
-    upperbound <- upperbound/population
+    observed <- ifelse(population!=0,observed/population,0)
+    upperbound <- ifelse(population!=0,upperbound/population,0)
     if (ylab == "No. infected") { ylab <- "Proportion infected" }
   }
   
@@ -388,6 +499,11 @@ plot.sts.time.one <- function(x, k=1, domany=FALSE,ylim=NULL,xaxis.years=TRUE, x
   y.points <- as.vector(t(cbind(0, observed, observed, 0, NA)))
   polygon(x.points,y.points,col=col[1],border=col[2],lwd=lwd[1])
 
+  #Draw upper bound once more in case the polygons are filled
+  if (!is.na(col[1])) {
+    lines(x=xstuff,y=ystuff,type=type,lty=lty[-c(1:2)],col=col[-c(1:2)],lwd=lwd[-c(1:2)],...)
+  }
+  
   #Draw outbreak symbols
   alarmIdx <- which(!is.na(alarm) & (alarm == 1))
   if (length(alarmIdx)>0) {
@@ -400,57 +516,13 @@ plot.sts.time.one <- function(x, k=1, domany=FALSE,ylim=NULL,xaxis.years=TRUE, x
     matpoints( stateIdx, rep(-1/20*ylim[2],length(stateIdx)), pch=outbreak.symbol$pch, col=outbreak.symbol$col,cex = outbreak.symbol$cex)
   }
 
-  #Label of x-axis 
-  if(xaxis.years){
-    if (x@freq ==52) {
-      # At which indices to put the "at" tick label. This will
-      # be exactly those week numbers where the new quarter begins: 1, 14, 27 and 40 + i*52.
-      # Note that week number and index is not the same due to the "firstweek" argument
-      weeks <- 1:length(observed) + (firstweek-1)
-      noYears <- ceiling(max(weeks)/52)
-      quarterStarts <- rep( (0:(noYears))*52, each=4) + rep( c(1,14,27,40), noYears+1)
-      weeks <- subset(weeks, !is.na(match(weeks,quarterStarts)))
-      weekIdx <- weeks - (firstweek-1)
-
-      # get the right year for each week
-      year <- weeks %/% 52 + startyear
-      # function to define the quarter order
-      quarterFunc <- function(i) { switch(i+1,"I","II","III","IV")}
-      # get the right number and order of quarter labels
-      quarter <- sapply( (weeks-1) %/% 13 %% 4, quarterFunc)
-
-      #construct the computed axis labels -- add quarters if xaxis.units is requested
-      if (xaxis.units) {
-        labels.week <- paste(year,"\n\n",quarter,sep="")
-      } else {
-        labels.week <- paste(year,sep="")
-      }
-
-      axis( side=1,line=1,labels=FALSE,at=c(1,length(observed)),lwd.ticks=0)
-      axis( at=weekIdx , labels=labels.week , side=1, line = 1 ,cex=cex)
-      axis( side=2 ,cex=cex)
-    } else { ##other frequency
-      #A label at each unit
-      myat.unit <- seq(firstweek,length.out=length(observed) )
-
-      # get the right year order
-      month <- (myat.unit-1) %% x@freq + 1
-      year <- (myat.unit - 1) %/% x@freq + startyear
-      #construct the computed axis labels -- add quarters if xaxis.units is requested
-      if (xaxis.units) {
-        mylabels.unit <- paste(year,"\n\n", (myat.unit-1) %% x@freq + 1,sep="")
-      } else {
-        mylabels.unit <- paste(year,sep="")
-      }
-      #Add axis
-      axis( at=(1:length(observed))  , labels=NA, side=1, line = 1 ,cex=cex)
-      axis( at=(1:length(observed))[month==1]  , labels=mylabels.unit[month==1] , side=1, line = 1 ,cex=cex)
-      #Bigger tick marks at the first unit
-      at <- (1:length(observed))[(myat.unit - 1) %% x@freq == 0]
-      axis( at=at  , labels=rep(NA,length(at)), side=1, line = 1 ,tcl=2*par()$tcl)
-      #2nd axis
-      axis( side=2 ,cex=cex)
-    }
+  #Label x-axis 
+  if(xaxis.years & axes) {
+    addFormattedXAxis(x, epochsAsDate, observed, firstweek,xaxis.units,cex)
+  }
+  #Label y-axis
+  if (axes) {
+    axis( side=2 ,cex=cex)
   }
 
   if(!is.null(legend.opts)) {
@@ -472,7 +544,7 @@ plot.sts.time.one <- function(x, k=1, domany=FALSE,ylim=NULL,xaxis.years=TRUE, x
 }
 
 
-plot.sts.alarm <- function(x, lvl=rep(1,nrow(x)), ylim=NULL,xaxis.years=TRUE, xaxis.units=TRUE, xlab="time", main=NULL, type="hhs",lty=c(1,1,2),col=c(1,1,4), outbreak.symbol = list(pch=3, col=3, cex=1),alarm.symbol=list(pch=24, col=2, cex=1),cex=1,cex.yaxis=1,...) {
+plot.sts.alarm <- function(x, lvl=rep(1,nrow(x)), ylim=NULL,xaxis.years=TRUE, xaxis.units=TRUE, epochsAsDate=x@epochAsDate, xlab="time", main=NULL, type="hhs",lty=c(1,1,2),col=c(1,1,4), outbreak.symbol = list(pch=3, col=3, cex=1),alarm.symbol=list(pch=24, col=2, cex=1),cex=1,cex.yaxis=1,...) {
 
   k <- 1
   #Extract slots -- depending on the algorithms: x@control$range
@@ -485,6 +557,7 @@ plot.sts.alarm <- function(x, lvl=rep(1,nrow(x)), ylim=NULL,xaxis.years=TRUE, xa
   firstweek <-  x@start[2]
   method <-     x@control$name
   disease <-    x@control$data
+  ylim <- c(0.5, ncol(x))
   
    ##### Handle the NULL arguments ######################################
   if (is.null(main)) {
@@ -521,57 +594,10 @@ plot.sts.alarm <- function(x, lvl=rep(1,nrow(x)), ylim=NULL,xaxis.years=TRUE, xa
 
   #Label of x-axis 
   if(xaxis.years){
-    if (x@freq ==52) {
-            # At which indices to put the "at" tick label. This will
-      # be exactly those week numbers where the new quarter begins: 1, 14, 27 and 40 + i*52.
-      # Note that week number and index is not the same due to the "firstweek" argument
-      weeks <- 1:length(observed) + (firstweek-1)
-      noYears <- ceiling(max(weeks)/52)
-      quarterStarts <- rep( (0:(noYears))*52, each=4) + rep( c(1,14,27,40), noYears+1)
-      weeks <- subset(weeks, !is.na(pmatch(weeks,quarterStarts)))
-      weekIdx <- weeks - (firstweek-1)
-
-      # get the right year for each week
-      year <- weeks %/% 52 + startyear
-      # function to define the quarter order
-      quarterFunc <- function(i) { switch(i+1,"I","II","III","IV")}
-      # get the right number and order of quarter labels
-      quarter <- sapply( (weeks-1) %/% 13 %% 4, quarterFunc)
-
-      #construct the computed axis labels -- add quarters if xaxis.units is requested
-      if (xaxis.units) {
-        labels.week <- paste(year,"\n\n",quarter,sep="")
-      } else {
-        labels.week <- paste(year,sep="")
-      }
-
-      axis( side=1,line=1,labels=FALSE,at=c(1,length(observed)),lwd.ticks=0)
-      axis( at=weekIdx , labels=labels.week , side=1, line = 1 ,cex=cex)
-      axis( side=2, at=1:ncol(x),cex.axis=cex.yaxis, labels=colnames(x),las=2)
-    } else { ##other frequency
-      #A label at each unit
-      myat.unit <- seq(firstweek,length.out=length(observed) )
-
-      # get the right year order
-      month <- (myat.unit-1) %% x@freq + 1
-      year <- (myat.unit - 1) %/% x@freq + startyear
-      #construct the computed axis labels -- add quarters if xaxis.units is requested
-      if (xaxis.units) {
-        mylabels.unit <- paste(year,"\n\n", (myat.unit-1) %% x@freq + 1,sep="")
-      } else {
-        mylabels.unit <- paste(year,sep="")
-      }
-      #Add axis
-      axis( at=(1:length(observed))  , labels=NA, side=1, line = 1 ,cex=cex)
-      axis( at=(1:length(observed))[month==1]  , labels=mylabels.unit[month==1] , side=1, line = 1 ,cex=cex)
-      #Bigger tick marks at the first unit
-      at <- (1:length(observed))[(myat.unit - 1) %% x@freq == 0]
-      axis( at=at  , labels=rep(NA,length(at)), side=1, line = 1 ,tcl=2*par()$tcl)
-      #2nd axis -- this is the important part
-      axis( side=2, at=1:ncol(x),cex.axis=cex.yaxis, labels=colnames(x),las=2)
-      #axis( side=2 ,cex=cex)
-    }
+    addFormattedXAxis(x, epochsAsDate, observed, firstweek,xaxis.units,cex)
   }
+  axis( side=2, at=1:ncol(x),cex.axis=cex.yaxis, labels=colnames(x),las=2)
+
 
   #Draw all alarms
   for (i in 1:nrow(x)) {
@@ -616,7 +642,7 @@ plot.sts.time <- function(x, type, method=x@control$name, disease=x@control$data
   state <- x@state
   alarm <- x@alarm
   population <- x@populationFrac
-  binaryTS <- sum( population > 1 ) > 1
+  binaryTS <- x@multinomialTS
   
   #univariate timeseries ?
   if(is.vector(observed))
@@ -629,7 +655,11 @@ plot.sts.time <- function(x, type, method=x@control$name, disease=x@control$data
 
   if (binaryTS) {
     pi <-  ifelse(population == 0,  0,observed / population)
-    un <-  ifelse(population == 0, 0, x@upperbound / population)
+    if (identical(dim(x@upperbound), population)) {
+      un <-  ifelse(population == 0, 0, x@upperbound / population)
+    } else {
+      un <- 0
+    }
     max <-  max(max(pi),max(un),na.rm=TRUE)
   } else {
     max <-  max(max(observed),max(x@upperbound),na.rm=TRUE)
@@ -946,15 +976,50 @@ setMethod( "show", "sts", function( object ){
   print( head(object@neighbourhood,n))
 } )
 
+######################################################################
+#Method to convert sts object to a data frame suitable for regression
+#Params:
+# row.names - from generic R function
+# optional  - from generic R function
+# freqByWeek -- if TRUE use information in week (supposed to be Dates)
+#               to freq (e.g. used for regression model)
+######################################################################
 
-#Method to convert sts object to a data frame
-setMethod("as.data.frame", signature(x="sts"), function(x,row.names = NULL, optional = FALSE,...) {
-  res <- data.frame("observed"=x@observed, "week"=x@week, "state"=x@state, "alarm"=x@alarm)
+setMethod("as.data.frame", signature(x="sts"), function(x,row.names = NULL, optional = FALSE, ...) {
+  #Convert object to data frame and give names
+  res <- data.frame("observed"=x@observed, "epoch"=x@week, "state"=x@state, "alarm"=x@alarm,"population"=x@populationFrac)
+
+  if (ncol(x) > 1) {
+    colnames(res) <-  c(paste("observed.",colnames(x@observed),sep=""),"epoch",
+                        paste("state.",colnames(x@observed),sep=""),
+                        paste("alarm.",colnames(x@observed),sep=""),
+                        paste("population.",colnames(x@observed),sep=""))
+  } else {
+      colnames(res) <-  c("observed","epoch","state","alarm","population")
+  }
+  
+  #Add a column denoting the number of week
+  if (x@epochAsDate) {
+    #Convert to date
+    date <- as.Date(x@week, origin="1970-01-01")
+    epochStr <- switch( as.character(x@freq), 
+                       "12" = "%m",
+                       "52" =  "%V",
+                       "365" = "%j")
+                       
+    #Find out how many epochs there are each year
+    years <- unique(as.numeric(format(date,"%Y")))
+    dummyDates <- as.Date(paste(rep(years,each=6),"-12-",26:31,sep=""))
+    maxEpoch <- tapply( as.numeric(format(dummyDates, epochStr)), rep(years,each=6), max)
+    #Assign this to result
+    res$freq <- maxEpoch[pmatch(format(date,"%Y"),names(maxEpoch),duplicates.ok=TRUE)]
+    res$epochInPeriod <- as.numeric(format(date,epochStr)) / res$freq
+  } else {
+    #Otherwise just replicate the fixed frequency
+    res$freq <- x@freq
+    res$epochInPeriod <- x@week %% res$freq
+  }
   
-  colnames(res) <-  c(paste("observed.",colnames(x@observed),sep=""),"week",
-                      paste("state.",colnames(x@observed),sep=""),
-                      paste("alarm.",colnames(x@observed),sep=""))
-
   return(res)
-          
 })
+
